@@ -257,7 +257,7 @@ void screenclear(void)
   compute_cmdrow();
   msg_row = cmdline_row;  // put cursor on last line for messages
   msg_col = 0;
-  msg_scrolled = 0;  // can't scroll back
+  msg_reset_scroll();     // can't scroll back
   msg_didany = false;
   msg_didout = false;
   if (HL_ATTR(HLF_MSG) > 0 && msg_use_grid() && msg_grid.chars) {
@@ -268,12 +268,19 @@ void screenclear(void)
   }
 }
 
+/// Unlike cmdline "one_key" prompts, the message part of the prompt is not stored
+/// to be re-emitted: avoid clearing the prompt from the message grid.
+static bool cmdline_number_prompt(void)
+{
+  return !ui_has(kUIMessages) && State == MODE_CMDLINE && get_cmdline_info()->mouse_used != NULL;
+}
+
 /// Set dimensions of the Nvim application "screen".
 void screen_resize(int width, int height)
 {
   // Avoid recursiveness, can happen when setting the window size causes
   // another window-changed signal.
-  if (updating_screen || resizing_screen) {
+  if (updating_screen || resizing_screen || cmdline_number_prompt()) {
     return;
   }
 
@@ -350,7 +357,7 @@ void screen_resize(int width, int height)
   resizing_autocmd = false;
   redraw_all_later(UPD_CLEAR);
 
-  if (State != MODE_ASKMORE && State != MODE_EXTERNCMD && State != MODE_CONFIRM) {
+  if (State != MODE_ASKMORE && State != MODE_EXTERNCMD) {
     screenclear();
   }
 
@@ -366,8 +373,11 @@ void screen_resize(int width, int height)
     // - While editing the command line, only redraw that. TODO: lies
     // - in Ex mode, don't redraw anything.
     // - Otherwise, redraw right now, and position the cursor.
-    if (State == MODE_ASKMORE || State == MODE_EXTERNCMD || State == MODE_CONFIRM
-        || exmode_active) {
+    if (State == MODE_ASKMORE || State == MODE_EXTERNCMD || exmode_active
+        || (State == MODE_CMDLINE && get_cmdline_info()->one_key)) {
+      if (State == MODE_CMDLINE) {
+        update_screen();
+      }
       if (msg_grid.chars) {
         msg_grid_validate();
       }
@@ -453,7 +463,7 @@ int update_screen(void)
 
   // Postpone the redrawing when it's not needed and when being called
   // recursively.
-  if (!redrawing() || updating_screen) {
+  if (!redrawing() || updating_screen || cmdline_number_prompt()) {
     return FAIL;
   }
 
@@ -492,7 +502,7 @@ int update_screen(void)
   }
 
   // if the screen was scrolled up when displaying a message, scroll it down
-  if (msg_scrolled || msg_grid_invalid) {
+  if ((msg_scrolled || msg_grid_invalid) && !cmdline_number_prompt()) {
     clear_cmdline = true;
     int valid = MAX(Rows - msg_scrollsize(), 0);
     if (msg_grid.chars) {
@@ -597,6 +607,11 @@ int update_screen(void)
     curwin->w_redr_type = UPD_NOT_VALID;
   }
 
+  if (curwin->w_redr_type == UPD_INVERTED) {
+    // Update w_curswant so that the end of Visual selection is correct.
+    update_curswant();
+  }
+
   // Redraw the tab pages line if needed.
   if (redraw_tabline || type >= UPD_NOT_VALID) {
     update_window_hl(curwin, type >= UPD_NOT_VALID);
@@ -694,6 +709,7 @@ int update_screen(void)
   if (still_may_intro) {
     intro_message(false);
   }
+  repeat_message();
 
   decor_providers_invoke_end();
 
