@@ -147,7 +147,6 @@ struct TUIData {
   char *space_buf;
   size_t space_buf_len;
   bool stopped;
-  int seen_error_exit;
   int width;
   int height;
   bool rgb;
@@ -170,7 +169,6 @@ void tui_start(TUIData **tui_p, int *width, int *height, char **term, bool *rgb)
   tui->is_starting = true;
   tui->screenshot = NULL;
   tui->stopped = false;
-  tui->seen_error_exit = 0;
   tui->loop = &main_loop;
   tui->url = -1;
 
@@ -380,6 +378,7 @@ static void terminfo_start(TUIData *tui)
   tui->unibi_ext.reset_scroll_region = -1;
   tui->unibi_ext.set_cursor_style = -1;
   tui->unibi_ext.reset_cursor_style = -1;
+  tui->unibi_ext.set_title = -1;
   tui->unibi_ext.set_underline_style = -1;
   tui->unibi_ext.set_underline_color = -1;
   tui->unibi_ext.sync = -1;
@@ -389,10 +388,12 @@ static void terminfo_start(TUIData *tui)
 
   const char *term = os_getenv("TERM");
 #ifdef MSWIN
-  os_tty_guess_term(&term, tui->out_fd);
-  os_setenv("TERM", term, 1);
-  // Old os_getenv() pointer is invalid after os_setenv(), fetch it again.
-  term = os_getenv("TERM");
+  const char *guessed_term = NULL;
+  os_tty_guess_term(&guessed_term, tui->out_fd);
+  if (term == NULL && guessed_term != NULL) {
+    term = xstrdup(guessed_term);
+    os_setenv("TERM", guessed_term, 1);
+  }
 #endif
 
   // Set up unibilium/terminfo.
@@ -558,14 +559,16 @@ static void terminfo_disable(TUIData *tui)
 /// Disable the alternate screen and prepare for the TUI to close.
 static void terminfo_stop(TUIData *tui)
 {
-  if (ui_client_exit_status == 0) {
-    ui_client_exit_status = tui->seen_error_exit;
+  if (ui_client_exit_status == 0 && ui_client_error_exit > 0) {
+    ui_client_exit_status = ui_client_error_exit;
   }
 
-  // if nvim exited with nonzero status, without indicated this was an
+  // If Nvim exited with nonzero status, without indicating this was an
   // intentional exit (like `:1cquit`), it likely was an internal failure.
-  // Don't clobber the stderr error message in this case.
-  if (ui_client_exit_status == tui->seen_error_exit) {
+  // Don't clobber the stderr error message in this case. #21608
+  if (ui_client_exit_status == MAX(ui_client_error_exit, 0)) {
+    // Position the cursor on the last screen line, below all the text
+    cursor_goto(tui, tui->height - 1, 0);
     // Exit alternate screen.
     unibi_out(tui, unibi_exit_ca_mode);
   }
@@ -604,12 +607,6 @@ static void tui_terminal_after_startup(TUIData *tui)
   flush_buf(tui);
 }
 
-void tui_error_exit(TUIData *tui, Integer status)
-  FUNC_ATTR_NONNULL_ALL
-{
-  tui->seen_error_exit = (int)status;
-}
-
 void tui_stop(TUIData *tui)
   FUNC_ATTR_NONNULL_ALL
 {
@@ -650,8 +647,6 @@ static void tui_terminal_stop(TUIData *tui)
   FUNC_ATTR_NONNULL_ALL
 {
   tinput_stop(&tui->input);
-  // Position the cursor on the last screen line, below all the text
-  cursor_goto(tui, tui->height - 1, 0);
   terminfo_stop(tui);
 }
 
@@ -1640,7 +1635,7 @@ static void tui_suspend_cb(TUIData *tui)
 
 void tui_set_title(TUIData *tui, String title)
 {
-  if (!unibi_get_ext_str(tui->ut, (unsigned)tui->unibi_ext.set_title)) {
+  if (tui->unibi_ext.set_title == -1) {
     return;
   }
   if (title.size > 0) {
@@ -1665,12 +1660,16 @@ void tui_set_icon(TUIData *tui, String icon)
 
 void tui_screenshot(TUIData *tui, String path)
 {
+  FILE *f = fopen(path.data, "w");
+  if (f == NULL) {
+    return;
+  }
+
   UGrid *grid = &tui->grid;
   flush_buf(tui);
   grid->row = 0;
   grid->col = 0;
 
-  FILE *f = fopen(path.data, "w");
   tui->screenshot = f;
   fprintf(f, "%d,%d\n", grid->height, grid->width);
   unibi_out(tui, unibi_clear_screen);
