@@ -27,7 +27,6 @@ local sleep = vim.uv.sleep
 local startswith = vim.startswith
 local write_file = t.write_file
 local api = n.api
-local alter_slashes = n.alter_slashes
 local is_os = t.is_os
 local dedent = t.dedent
 local tbl_map = vim.tbl_map
@@ -40,22 +39,15 @@ local testlog = 'Xtest-startupspec-log'
 describe('startup', function()
   it('--clean', function()
     clear()
-    ok(
-      string.find(
-        alter_slashes(api.nvim_get_option_value('runtimepath', {})),
-        fn.stdpath('config'),
-        1,
-        true
-      ) ~= nil
+    matches(
+      vim.pesc(t.fix_slashes(fn.stdpath('config'))),
+      t.fix_slashes(api.nvim_get_option_value('runtimepath', {}))
     )
+
     clear('--clean')
     ok(
-      string.find(
-        alter_slashes(api.nvim_get_option_value('runtimepath', {})),
-        fn.stdpath('config'),
-        1,
-        true
-      ) == nil
+      not t.fix_slashes(api.nvim_get_option_value('runtimepath', {}))
+        :match(vim.pesc(t.fix_slashes(fn.stdpath('config'))))
     )
   end)
 
@@ -63,8 +55,10 @@ describe('startup', function()
     clear()
     local screen
     screen = Screen.new(84, 3)
-    screen:attach()
-    fn.termopen({ nvim_prog, '-u', 'NONE', '--server', eval('v:servername'), '--remote-ui' })
+    fn.jobstart(
+      { nvim_prog, '-u', 'NONE', '--server', eval('v:servername'), '--remote-ui' },
+      { term = true }
+    )
     screen:expect([[
       ^Cannot attach UI of :terminal child to its parent. (Unset $NVIM to skip this check) |
                                                                                           |*2
@@ -83,32 +77,18 @@ describe('startup', function()
   end)
 
   it('--startuptime does not crash on error #31125', function()
-    eq(
-      "E484: Can't open file .",
-      fn.system({
-        nvim_prog,
-        '-u',
-        'NONE',
-        '-i',
-        'NONE',
-        '--headless',
-        '--startuptime',
-        '.',
-        '-c',
-        '42cquit',
-      })
-    )
-    eq(42, api.nvim_get_vvar('shell_error'))
+    local p = n.spawn_wait('--startuptime', '.', '-c', '42cquit')
+    eq("E484: Can't open file .", p.stderr)
+    eq(42, p.status)
   end)
 
   it('-D does not hang #12647', function()
     clear()
     local screen
     screen = Screen.new(60, 7)
-    screen:attach()
     -- not the same colors on windows for some reason
     screen._default_attr_ids = nil
-    local id = fn.termopen({
+    local id = fn.jobstart({
       nvim_prog,
       '-u',
       'NONE',
@@ -118,6 +98,7 @@ describe('startup', function()
       'set noruler',
       '-D',
     }, {
+      term = true,
       env = {
         VIMRUNTIME = os.getenv('VIMRUNTIME'),
       },
@@ -131,6 +112,13 @@ describe('startup', function()
                                                                   |*2
     ]])
   end)
+
+  it(':filetype detect enables filetype detection with -u NONE', function()
+    clear()
+    eq('filetype detection:OFF  plugin:OFF  indent:OFF', exec_capture('filetype'))
+    command('filetype detect')
+    eq('filetype detection:ON  plugin:OFF  indent:OFF', exec_capture('filetype'))
+  end)
 end)
 
 describe('startup', function()
@@ -148,13 +136,18 @@ describe('startup', function()
       vim.list_extend(args, { '-l', (script or 'test/functional/fixtures/startup.lua') })
       vim.list_extend(args, lua_args or {})
       local out = fn.system(args, input):gsub('\r\n', '\n')
-      return eq(dedent(expected), out)
+      if type(expected) == 'function' then
+        return expected(out)
+      else
+        return eq(dedent(expected), out)
+      end
     end
 
     it('failure modes', function()
       -- nvim -l <empty>
-      matches('nvim%.?e?x?e?: Argument missing after: "%-l"', fn.system({ nvim_prog, '-l' }))
-      eq(1, eval('v:shell_error'))
+      local proc = n.spawn_wait('-l')
+      matches('nvim%.?e?x?e?: Argument missing after: "%-l"', proc.stderr)
+      eq(1, proc.status)
     end)
 
     it('os.exit() sets Nvim exitcode', function()
@@ -181,12 +174,12 @@ describe('startup', function()
     end)
 
     it('Lua-error sets Nvim exitcode', function()
+      local proc = n.spawn_wait('-l', 'test/functional/fixtures/startup-fail.lua')
+      matches('E5113: .* my pearls!!', proc:output())
+      eq(0, proc.signal)
+      eq(1, proc.status)
+
       eq(0, eval('v:shell_error'))
-      matches(
-        'E5113: .* my pearls!!',
-        fn.system({ nvim_prog, '-l', 'test/functional/fixtures/startup-fail.lua' })
-      )
-      eq(1, eval('v:shell_error'))
       matches(
         'E5113: .* %[string "error%("whoa"%)"%]:1: whoa',
         fn.system({ nvim_prog, '-l', '-' }, 'error("whoa")')
@@ -280,10 +273,8 @@ describe('startup', function()
 
       -- nvim <vim args> -l foo.lua <vim args>
       assert_l_out(
-        -- luacheck: ignore 611 (Line contains only whitespaces)
         [[
             wrap
-          
           bufs:
           nvim args: 7
           lua args: { "-c", "set wrap?",
@@ -296,14 +287,30 @@ describe('startup', function()
       eq(0, eval('v:shell_error'))
     end)
 
-    it('disables swapfile/shada/config/plugins', function()
+    it('disables swapfile/shada/config/plugins unless overridden', function()
+      local script = [[print(('updatecount=%d shadafile=%s loadplugins=%s scripts=%d'):format(
+                       vim.o.updatecount, vim.o.shadafile, tostring(vim.o.loadplugins), math.max(1, #vim.fn.getscriptinfo())))]]
+      finally(function()
+        os.remove('Xtest_shada')
+      end)
+
       assert_l_out(
         'updatecount=0 shadafile=NONE loadplugins=false scripts=1\n',
         nil,
         nil,
         '-',
-        [[print(('updatecount=%d shadafile=%s loadplugins=%s scripts=%d'):format(
-          vim.o.updatecount, vim.o.shadafile, tostring(vim.o.loadplugins), math.max(1, #vim.fn.getscriptinfo())))]]
+        script
+      )
+
+      -- User can override.
+      assert_l_out(
+        function(out)
+          return matches('updatecount=99 shadafile=Xtest_shada loadplugins=true scripts=2%d\n', out)
+        end,
+        { '+set updatecount=99', '-i', 'Xtest_shada', '+set loadplugins', '-u', 'NORC' },
+        nil,
+        '-',
+        script
       )
     end)
   end)
@@ -347,9 +354,8 @@ describe('startup', function()
   it('with --embed: has("ttyin")==0 has("ttyout")==0', function()
     local screen = Screen.new(25, 3)
     -- Remote UI connected by --embed.
-    screen:attach()
     -- TODO: a lot of tests in this file already use the new default color scheme.
-    -- once we do the batch update of tests to use it, remove this workarond
+    -- once we do the batch update of tests to use it, remove this workaround
     screen._default_attr_ids = nil
     command([[echo has('ttyin') has('ttyout')]])
     screen:expect([[
@@ -361,13 +367,12 @@ describe('startup', function()
 
   it('in a TTY: has("ttyin")==1 has("ttyout")==1', function()
     local screen = Screen.new(25, 4)
-    screen:attach()
     screen._default_attr_ids = nil
     if is_os('win') then
       command([[set shellcmdflag=/s\ /c shellxquote=\"]])
     end
     -- Running in :terminal
-    fn.termopen({
+    fn.jobstart({
       nvim_prog,
       '-u',
       'NONE',
@@ -378,6 +383,7 @@ describe('startup', function()
       '-c',
       'echo has("ttyin") has("ttyout")',
     }, {
+      term = true,
       env = {
         VIMRUNTIME = os.getenv('VIMRUNTIME'),
       },
@@ -400,13 +406,14 @@ describe('startup', function()
       os.remove('Xtest_startup_ttyout')
     end)
     -- Running in :terminal
-    fn.termopen(
+    fn.jobstart(
       (
         [["%s" -u NONE -i NONE --cmd "%s"]]
         .. [[ -c "call writefile([has('ttyin'), has('ttyout')], 'Xtest_startup_ttyout')"]]
         .. [[ -c q | cat -v]]
       ):format(nvim_prog, nvim_set),
       {
+        term = true,
         env = {
           VIMRUNTIME = os.getenv('VIMRUNTIME'),
         },
@@ -431,7 +438,7 @@ describe('startup', function()
       os.remove('Xtest_startup_ttyout')
     end)
     -- Running in :terminal
-    fn.termopen(
+    fn.jobstart(
       (
         [[echo foo | ]] -- Input from a pipe.
         .. [["%s" -u NONE -i NONE --cmd "%s"]]
@@ -439,6 +446,7 @@ describe('startup', function()
         .. [[ -c q -- -]]
       ):format(nvim_prog, nvim_set),
       {
+        term = true,
         env = {
           VIMRUNTIME = os.getenv('VIMRUNTIME'),
         },
@@ -456,19 +464,19 @@ describe('startup', function()
   it('input from pipe (implicit) #7679', function()
     clear({ env = { NVIM_LOG_FILE = testlog } })
     local screen = Screen.new(25, 4)
-    screen:attach()
     screen._default_attr_ids = nil
     if is_os('win') then
       command([[set shellcmdflag=/s\ /c shellxquote=\"]])
     end
     -- Running in :terminal
-    fn.termopen(
+    fn.jobstart(
       (
         [[echo foo | ]]
         .. [["%s" -u NONE -i NONE --cmd "%s"]]
         .. [[ -c "echo has('ttyin') has('ttyout')"]]
       ):format(nvim_prog, nvim_set),
       {
+        term = true,
         env = {
           VIMRUNTIME = os.getenv('VIMRUNTIME'),
         },
@@ -585,19 +593,21 @@ describe('startup', function()
     eq('  encoding=utf-8\n', fn.system({ nvim_prog, '-n', '-es' }, { 'set encoding', '' }))
   end)
 
-  it('-es/-Es disables swapfile, user config #8540', function()
+  it('-es/-Es disables swapfile/shada/config #8540', function()
     for _, arg in ipairs({ '-es', '-Es' }) do
       local out = fn.system({
         nvim_prog,
         arg,
-        '+set swapfile? updatecount? shadafile?',
+        '+set updatecount? shadafile? loadplugins?',
         '+put =map(getscriptinfo(), {-> v:val.name})',
         '+%print',
       })
       local line1 = string.match(out, '^.-\n')
       -- updatecount=0 means swapfile was disabled.
-      eq('  swapfile  updatecount=0  shadafile=\n', line1)
-      -- Standard plugins were loaded, but not user config.
+      eq('  updatecount=0  shadafile=NONE  loadplugins\n', line1)
+      -- Standard plugins were loaded, but not user config. #31878
+      local nrlines = #vim.split(out, '\n')
+      ok(nrlines > 20, '>20', nrlines)
       ok(string.find(out, 'man.lua') ~= nil)
       ok(string.find(out, 'init.vim') == nil)
     end
@@ -606,24 +616,23 @@ describe('startup', function()
   it('fails on --embed with -es/-Es/-l', function()
     matches(
       'nvim[.exe]*: %-%-embed conflicts with %-es/%-Es/%-l',
-      fn.system({ nvim_prog, '--embed', '-es' })
+      n.spawn_wait('--embed', '-es').stderr
     )
     matches(
       'nvim[.exe]*: %-%-embed conflicts with %-es/%-Es/%-l',
-      fn.system({ nvim_prog, '--embed', '-Es' })
+      n.spawn_wait('--embed', '-Es').stderr
     )
     matches(
       'nvim[.exe]*: %-%-embed conflicts with %-es/%-Es/%-l',
-      fn.system({ nvim_prog, '--embed', '-l', 'foo.lua' })
+      n.spawn_wait('--embed', '-l', 'foo.lua').stderr
     )
   end)
 
   it('ENTER dismisses early message #7967', function()
     local screen
     screen = Screen.new(60, 6)
-    screen:attach()
     screen._default_attr_ids = nil
-    local id = fn.termopen({
+    local id = fn.jobstart({
       nvim_prog,
       '-u',
       'NONE',
@@ -634,6 +643,7 @@ describe('startup', function()
       '--cmd',
       'let g:foo = g:bar',
     }, {
+      term = true,
       env = {
         VIMRUNTIME = os.getenv('VIMRUNTIME'),
       },
@@ -698,20 +708,8 @@ describe('startup', function()
   end)
 
   it('get command line arguments from v:argv', function()
-    local out = fn.system({
-      nvim_prog,
-      '-u',
-      'NONE',
-      '-i',
-      'NONE',
-      '--headless',
-      '--cmd',
-      nvim_set,
-      '-c',
-      [[echo v:argv[-1:] len(v:argv) > 1]],
-      '+q',
-    })
-    eq("['+q'] 1", out)
+    local p = n.spawn_wait('--cmd', nvim_set, '-c', [[echo v:argv[-1:] len(v:argv) > 1]], '+q')
+    eq("['+q'] 1", p.stderr)
   end)
 end)
 
@@ -719,7 +717,6 @@ describe('startup', function()
   it('-e/-E interactive #7679', function()
     clear('-e')
     local screen = Screen.new(25, 3)
-    screen:attach()
     feed("put ='from -e'<CR>")
     screen:expect([[
       :put ='from -e'          |
@@ -729,7 +726,6 @@ describe('startup', function()
 
     clear('-E')
     screen = Screen.new(25, 3)
-    screen:attach()
     feed("put ='from -E'<CR>")
     screen:expect([[
       :put ='from -E'          |
@@ -739,9 +735,8 @@ describe('startup', function()
   end)
 
   it('-e sets ex mode', function()
-    local screen = Screen.new(25, 3)
     clear('-e')
-    screen:attach()
+    local screen = Screen.new(25, 3)
     -- Verify we set the proper mode both before and after :vi.
     feed('put =mode(1)<CR>vi<CR>:put =mode(1)<CR>')
     screen:expect([[
@@ -766,7 +761,8 @@ describe('startup', function()
             \   'row': 3,
             \   'col': 3
             \ }
-      autocmd WinEnter * call nvim_open_win(bufnr, v:false, config)]]
+      autocmd WinEnter * let g:float_win = nvim_open_win(bufnr, v:false, config)
+    ]]
     )
     finally(function()
       os.remove('Xdiff.vim')
@@ -779,6 +775,45 @@ describe('startup', function()
     eq(false, api.nvim_get_option_value('diff', { win = float_win }))
   end)
 
+  it('still opens arglist windows if init only opens floating windows', function()
+    write_file(
+      'Xfloat.vim',
+      [[
+      let config = {
+            \   'relative': 'editor',
+            \   'focusable': v:false,
+            \   'width': 1,
+            \   'height': 1,
+            \   'row': 1,
+            \   'col': 1
+            \ }
+      call nvim_open_win(0, v:false, config)
+    ]]
+    )
+    finally(function()
+      os.remove('Xfloat.vim')
+    end)
+    clear { args = { '-u', 'Xfloat.vim', '-d', 'Xfloat.vim', 'foo' } }
+    local screen = Screen.new(40, 5)
+    screen:add_extra_attr_ids({
+      [101] = {
+        bold = true,
+        background = Screen.colors.LightBlue,
+        foreground = Screen.colors.Brown,
+      },
+      [102] = { foreground = Screen.colors.SlateBlue, background = Screen.colors.LightBlue },
+      [103] = { foreground = Screen.colors.Magenta, background = Screen.colors.LightBlue },
+      [104] = { foreground = Screen.colors.DarkCyan, background = Screen.colors.LightBlue },
+    })
+    screen:expect([[
+      {7:  }{101:^let}{22: config }{101:=}{22: }{102:{}{22:                      }│{23:-}|
+      {7: }{101:l}{22:      }{102:\}{22:   }{103:'relative'}{22:: }{103:'editor'}{22:,     }│{23:-}|
+      {7:  }{22:      }{102:\}{22:   }{103:'focusable'}{22:: }{104:v:false}{22:,     }│ |
+      {3:Xfloat.vim                             }{2:<}|
+                                              |
+    ]])
+  end)
+
   it('does not crash if --embed is given twice', function()
     clear { args = { '--embed' } }
     assert_alive()
@@ -793,21 +828,12 @@ describe('startup', function()
   it("sets 'shortmess' when loading other tabs", function()
     clear({ args = { '-p', 'a', 'b', 'c' } })
     local screen = Screen.new(25, 4)
-    screen:attach()
-    screen:expect({
-      grid = [[
-        {1: a }{2: b  c }{3:               }{2:X}|
-        ^                         |
-        {4:~                        }|
-                                 |
-          ]],
-      attr_ids = {
-        [1] = { bold = true },
-        [2] = { background = Screen.colors.LightGrey, underline = true },
-        [3] = { reverse = true },
-        [4] = { bold = true, foreground = Screen.colors.Blue1 },
-      },
-    })
+    screen:expect([[
+      {5: a }{24: b  c }{2:               }{24:X}|
+      ^                         |
+      {1:~                        }|
+                               |
+    ]])
   end)
 end)
 
@@ -1114,6 +1140,7 @@ describe('user config init', function()
           string.format(
             [[
           vim.g.exrc_file = "%s"
+          vim.g.exrc_path = debug.getinfo(1, 'S').source:sub(2)
         ]],
             exrc_path
           )
@@ -1124,6 +1151,7 @@ describe('user config init', function()
           string.format(
             [[
           let g:exrc_file = "%s"
+          " let g:exrc_path = ??
         ]],
             exrc_path
           )
@@ -1156,9 +1184,9 @@ describe('user config init', function()
         eq('---', eval('g:exrc_file'))
 
         local screen = Screen.new(50, 8)
-        screen:attach()
         screen._default_attr_ids = nil
-        fn.termopen({ nvim_prog }, {
+        fn.jobstart({ nvim_prog }, {
+          term = true,
           env = {
             VIMRUNTIME = os.getenv('VIMRUNTIME'),
           },
@@ -1167,7 +1195,7 @@ describe('user config init', function()
         -- `i` to enter Terminal mode, `a` to allow
         feed('ia')
         screen:expect([[
-                                                            |
+          ^                                                  |
           ~                                                 |*4
           [No Name]                       0,0-1          All|
                                                             |
@@ -1176,7 +1204,7 @@ describe('user config init', function()
         feed(':echo g:exrc_file<CR>')
         screen:expect(string.format(
           [[
-                                                            |
+          ^                                                  |
           ~                                                 |*4
           [No Name]                       0,0-1          All|
           %s%s|
@@ -1187,6 +1215,12 @@ describe('user config init', function()
         ))
 
         clear { args_rm = { '-u' }, env = xstateenv }
+        if string.find(exrc_path, '%.lua$') then
+          eq(
+            vim.fs.normalize(vim.fs.abspath(filename)),
+            vim.fs.normalize(vim.fs.abspath(eval('g:exrc_path')))
+          )
+        end
         -- The 'exrc' file is now trusted.
         eq(filename, eval('g:exrc_file'))
       end)
@@ -1344,31 +1378,59 @@ describe('runtime:', function()
   end)
 
   it("loads ftdetect/*.{vim,lua} respecting 'rtp' order", function()
-    local ftdetect_folder = table.concat({ xconfig, 'nvim', 'ftdetect' }, pathsep)
-    local after_ftdetect_folder = table.concat({ xconfig, 'nvim', 'after', 'ftdetect' }, pathsep)
+    local rtp_folder = table.concat({ xconfig, 'nvim' }, pathsep)
+    local after_rtp_folder = table.concat({ rtp_folder, 'after' }, pathsep)
+    local ftdetect_folder = table.concat({ rtp_folder, 'ftdetect' }, pathsep)
+    local after_ftdetect_folder = table.concat({ after_rtp_folder, 'ftdetect' }, pathsep)
     mkdir_p(ftdetect_folder)
     mkdir_p(after_ftdetect_folder)
     finally(function()
       rmdir(ftdetect_folder)
       rmdir(after_ftdetect_folder)
     end)
+    write_file(table.concat({ rtp_folder, 'scripts.vim' }, pathsep), [[let g:aseq ..= 'S']])
+    write_file(table.concat({ after_rtp_folder, 'scripts.vim' }, pathsep), [[let g:aseq ..= 's']])
     -- A .lua file is loaded after a .vim file if they only differ in extension.
     -- All files in after/ftdetect/ are loaded after all files in ftdetect/.
-    write_file(table.concat({ ftdetect_folder, 'new-ft.vim' }, pathsep), [[let g:seq ..= 'A']])
+    write_file(
+      table.concat({ ftdetect_folder, 'new-ft.vim' }, pathsep),
+      [[
+        let g:seq ..= 'A'
+        autocmd BufRead,BufNewFile FTDETECT let g:aseq ..= 'A'
+      ]]
+    )
     write_file(
       table.concat({ ftdetect_folder, 'new-ft.lua' }, pathsep),
-      [[vim.g.seq = vim.g.seq .. 'B']]
+      [[
+        vim.g.seq = vim.g.seq .. 'B'
+        vim.api.nvim_create_autocmd({ 'BufRead', 'BufNewFile' }, {
+          pattern = 'FTDETECT',
+          command = "let g:aseq ..= 'B'",
+        })
+      ]]
     )
     write_file(
       table.concat({ after_ftdetect_folder, 'new-ft.vim' }, pathsep),
-      [[let g:seq ..= 'a']]
+      [[
+        let g:seq ..= 'a'
+        autocmd BufRead,BufNewFile FTDETECT let g:aseq ..= 'a'
+      ]]
     )
     write_file(
       table.concat({ after_ftdetect_folder, 'new-ft.lua' }, pathsep),
-      [[vim.g.seq = vim.g.seq .. 'b']]
+      [[
+        vim.g.seq = vim.g.seq .. 'b'
+        vim.api.nvim_create_autocmd({ 'BufRead', 'BufNewFile' }, {
+          pattern = 'FTDETECT',
+          command = "let g:aseq ..= 'b'",
+        })
+      ]]
     )
     clear { args_rm = { '-u' }, args = { '--cmd', 'let g:seq = ""' }, env = xenv }
     eq('ABab', eval('g:seq'))
+    command('let g:aseq = ""')
+    command('edit FTDETECT')
+    eq('SsABab', eval('g:aseq'))
   end)
 end)
 
@@ -1404,8 +1466,7 @@ describe('inccommand on ex mode', function()
     clear()
     local screen
     screen = Screen.new(60, 10)
-    screen:attach()
-    local id = fn.termopen({
+    local id = fn.jobstart({
       nvim_prog,
       '-u',
       'NONE',
@@ -1416,23 +1477,22 @@ describe('inccommand on ex mode', function()
       '-E',
       'test/README.md',
     }, {
+      term = true,
       env = { VIMRUNTIME = os.getenv('VIMRUNTIME') },
     })
     fn.chansend(id, '%s/N')
-    screen:expect {
-      grid = [[
-      {1:^                                                            }|
-      {1:                                                            }|*6
-      {1:Entering Ex mode.  Type "visual" to go to Normal mode.      }|
-      {1::%s/N                                                       }|
-                                                                  |
-    ]],
-      attr_ids = {
-        [1] = {
-          background = Screen.colors.NvimDarkGrey2,
-          foreground = Screen.colors.NvimLightGrey2,
-        },
+    screen:add_extra_attr_ids({
+      [101] = {
+        background = Screen.colors.NvimDarkGrey2,
+        foreground = Screen.colors.NvimLightGrey2,
       },
-    }
+    })
+    screen:expect([[
+      {101:^                                                            }|
+      {101:                                                            }|*6
+      {101:Entering Ex mode.  Type "visual" to go to Normal mode.      }|
+      {101::%s/N                                                       }|
+                                                                  |
+    ]])
   end)
 end)
